@@ -16,7 +16,7 @@ use Rose::DB::Object::Metadata::Util qw(perl_hashref);
 use Rose::Object;
 our @ISA = qw(Rose::Object);
 
-our $VERSION = '0.721';
+our $VERSION = '0.73';
 
 use Rose::Object::MakeMethods::Generic
 (
@@ -32,6 +32,7 @@ use Rose::Object::MakeMethods::Generic
     'exclude_tables',
     'filter_tables',
     'pre_init_hook',
+    'post_init_hook',
     'module_dir',
   ],
 
@@ -340,8 +341,21 @@ sub _perl_class
 {
   my($class, $info, $args) = @_;
 
+  my $auto_load = $args->{'auto_load_related_classes'};
+  my $old_auto_load = $class->meta->auto_load_related_classes;
+
+  if(defined $auto_load)
+  {
+    $class->meta->auto_load_related_classes($auto_load);
+  }
+
   my $code = $class->meta->perl_class_definition(%$args);
 
+  if(defined $auto_load)
+  {
+    $class->meta->auto_load_related_classes($old_auto_load);
+  }
+
   if(!$info->{'init_db_in_base_class'} && $info->{'perl_init_db'})
   {
     my $init_db = $info->{'perl_init_db'};
@@ -366,8 +380,7 @@ sub _perl_base_class
   return<<"EOF";
 package $class;
 
-use Rose::DB::Object;
-our \@ISA = qw(Rose::DB::Object);
+use base 'Rose::DB::Object';
 $init_db
 1;
 EOF
@@ -395,22 +408,21 @@ package $class;
 
 use strict;
 
-use Rose::DB;
-our \@ISA = qw(Rose::DB);
+use base 'Rose::DB';
 
 __PACKAGE__->use_private_registry;
 
 __PACKAGE__->register_db
 (
-  domain => __PACKAGE__->default_domain,
-  type   => __PACKAGE__->default_type,
 $hash
 );
+
 1;
 EOF
 }
 
-sub default_pre_init_hook { }
+sub default_pre_init_hook  { }
+sub default_post_init_hook { }
 
 sub make_classes
 {
@@ -439,6 +451,9 @@ sub make_classes
   my $pre_init_hook = exists $args{'pre_init_hook'} ? 
     delete $args{'pre_init_hook'} : $self->pre_init_hook;
 
+  my $post_init_hook = exists $args{'post_init_hook'} ? 
+    delete $args{'post_init_hook'} : $self->post_init_hook;
+
   my $include = exists $args{'include_tables'} ? 
     delete $args{'include_tables'} : $self->include_tables;
 
@@ -626,7 +641,7 @@ sub make_classes
     $init_db = sub { $db_class->new };
 
     $extra_info->{'perl_init_db'} =
-      "use $db_class;\n" .
+      "use $db_class;\n\n" .
       "sub init_db { $db_class->new }";
   }
   else
@@ -639,7 +654,7 @@ sub make_classes
                             indent     => 0);
 
     $extra_info->{'perl_init_db'} = 
-      "use $db_class;\n" .
+      "use $db_class;\n\n" .
       "sub init_db { $db_class->new($hash) }";
   }
 
@@ -709,6 +724,9 @@ sub make_classes
     local $_ = $table;
     next  unless(!$filter || $filter->($table));
 
+    # Skip tables with no primary keys
+    next  unless($db->has_primary_key($table));
+
     my $obj_class = $class_prefix . $cm->table_to_class($table);
 
     # Set up the class
@@ -738,6 +756,22 @@ sub make_classes
 
     $meta->pre_init_hook($pre_init_hook);
 
+    if($post_init_hook)
+    {
+      if(ref $post_init_hook eq 'CODE')
+      {
+        $post_init_hook = [ $post_init_hook ];
+      }
+      elsif(ref $post_init_hook ne 'ARRAY')
+      {
+        Carp::croak "Invalid post_init_hook: $post_init_hook";
+      }
+    }
+
+    unshift(@$post_init_hook, sub { $self->default_post_init_hook(@_) });
+
+    $meta->post_init_hook($post_init_hook);
+
     $meta->table($table);
     $meta->convention_manager($cm_class->new);
 
@@ -995,13 +1029,9 @@ Get or set the L<schema|Rose::DB/schema> for the database connection.
 
 Get or set the L<username|Rose::DB/username> used to connect to the database.
 
-=item B<include_tables REGEX>
-
-Table names that do not match REGEX will be skipped by default during calls to the L<make_classes|/make_classes> method.
-
 =item B<exclude_tables REGEX>
 
-Table names that match REGEX will be skipped during calls to the L<make_classes|/make_classes> method.
+Table names that match REGEX will be skipped during calls to the L<make_classes|/make_classes> method.  Tables without primary keys are automatically skipped.
 
 =item B<filter_tables CODEREF>
 
@@ -1013,13 +1043,17 @@ This attribute should not be combined with the L<exclude_tables|/exclude_tables>
 
 Given the name of a L<Rose::DB::Object>-derived class, returns a class name for a L<Rose::DB::Object::Manager>-derived class to manage such objects.  The default implementation calls the L<auto_manager_class_name|Rose::DB::Object::ConventionManager/auto_manager_class_name> method on the L<convention_manager|/convention_manager> object.
 
+=item B<include_tables REGEX>
+
+Table names that do not match REGEX will be skipped by default during calls to the L<make_classes|/make_classes> method.  Tables without primary keys are automatically (and always) skipped.
+
 =item B<include_views BOOL>
 
 If true, database views will also be processed by default during calls to the L<make_classes|/make_classes> method.  Defaults to false.
 
 =item B<make_classes [PARAMS]>
 
-Automatically create L<Rose::DB::Object> and (optionally) L<Rose::DB::Object::Manager> subclasses for some or all of the tables in a database.  The process is controlled by the object attributes described above.  Optional name/value pairs passed to this method may override some of those values.  Valid PARAMS are:
+Automatically create L<Rose::DB::Object> and (optionally) L<Rose::DB::Object::Manager> subclasses for some or all of the tables in a database.  The class creation process is controlled by the loader object's attributes.  Optional name/value pairs passed to this method may override some of those values.  Valid PARAMS are:
 
 =over 4
 
@@ -1033,25 +1067,29 @@ The name of the L<Rose::DB>-derived class used to construct a L<db|/db> object i
 
 =item B<include_tables REGEX>
 
-Table names that do not match REGEX will be skipped.  Defaults to the value of the loader object's L<include_tables|/include_tables> attribute.
+Table names that do not match REGEX will be skipped.  Defaults to the value of the loader object's L<include_tables|/include_tables> attribute.  Tables without primary keys are automatically (and always) skipped.
 
 =item B<exclude_tables REGEX>
 
-Table names that match REGEX will be skipped.  Defaults to the value of the loader object's L<exclude_tables|/exclude_tables> attribute.
+Table names that match REGEX will be skipped.  Defaults to the value of the loader object's L<exclude_tables|/exclude_tables> attribute.  Tables without primary keys are automatically skipped.
 
 =item B<filter_tables CODEREF>
 
 A reference to a subroutine that takes a single table name argument and returns true if the table should be processed, false if it should be skipped.  The C<$_> variable will also be set to the table name before the call.  This parameter cannot be combined with the C<exclude_tables> or C<include_tables> options.
 
-Defaults to the value of the loader object's L<filter_tables|/filter_tables> attribute, provided that both the C<exclude_tables> and C<include_tables> values are undefined.
+Defaults to the value of the loader object's L<filter_tables|/filter_tables> attribute, provided that both the C<exclude_tables> and C<include_tables> values are undefined.  Tables without primary keys are automatically skipped.
 
 =item B<include_views BOOL>
 
 If true, database views will also be processed.  Defaults to the value of the loader object's L<include_views|/include_views> attribute.
 
+=item B<post_init_hook [ CODEREF | ARRAYREF ]>
+
+A reference to a subroutine or a reference to an array of code references that will be called just after each L<Rose::DB::Object>-derived class is L<initialize|Rose::DB::Object::Metadata/initialize>d.  Each referenced subroutine will be passed the class's L<metadata|Rose::DB::Object::Metadata> object plus any arguments to the L<initialize|Rose::DB::Object::Metadata/initialize> method.  Defaults to the value of the loader object's L<post_init_hook|/post_init_hook> attribute.
+
 =item B<pre_init_hook [ CODEREF | ARRAYREF ]>
 
-A reference to a subroutine or a reference to an array of code references that will be called just before each L<Rose::DB::Object>-derived class is L<initialize|Rose::DB::Object::Metadata/initialize>ed.  Each referenced subroutine will be passed the class's L<metdata|Rose::DB::Object::Metadata> object plus any arguments to the L<initialize|Rose::DB::Object::Metadata/initialize> method.  Defaults to the value of the loader object's L<pre_init_hook|/pre_init_hook> attribute.
+A reference to a subroutine or a reference to an array of code references that will be called just before each L<Rose::DB::Object>-derived class is L<initialize|Rose::DB::Object::Metadata/initialize>d.  Each referenced subroutine will be passed the class's L<metadata|Rose::DB::Object::Metadata> object plus any arguments to the L<initialize|Rose::DB::Object::Metadata/initialize> method.  Defaults to the value of the loader object's L<pre_init_hook|/pre_init_hook> attribute.
 
 =item B<with_foreign_keys BOOL>
 
